﻿"""
/***************************************************************************
Name                  : Mask 
Description          : Aide à la création de masque
Date                 : Feb/12 
copyright            : (C) 2011 by AEAG
email                : xavier.culos@eau-adour-garonne.fr 
todo: 

 add help file
 let's user change user defined parameters. 
 use a qml file to set masqk style. default qml inside plugin ? 
   Parameter 1: choose memory output or shp output + directory (default HOME directory)
   Parameter 2 : choose a qml file. Qml should be parsed to check if qymbology apply to polygon features

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
# Import the PyQt and QGIS libraries
import os
from PyQt4.QtCore import * 
from PyQt4.QtGui import *
from qgis.core import *

# Initialize Qt resources from file resources.py
import resources

_fromUtf8 = lambda s: (s.decode("utf-8").encode("latin-1")) if s else s
_toUtf8 = lambda s: s.decode("latin-1").encode("utf-8") if s else s

aeag_mask_instance = None

def do(workOnSelectedFeatures=True, extent=None):
    global aeag_mask_instance
    aeag_mask_instance.do_aeag_mask(workOnSelectedFeatures, extent);

class aeag_mask: 

    def __init__(self, iface):
        global aeag_mask_instance
        # Save reference to the QGIS interface
        self.iface = iface
        self.toolBar = None
        self.act_aeag_mask = None
        self.act_aeag_toolbar_help = None
        self.canvas = self.iface.mapCanvas()
        self.active = False
        self.read()
        aeag_mask_instance = self
      
    def store(self):
        s = QSettings() 
    
    def read(self):
        s = QSettings()

    def initGui(self):  
        self.toolBar = self.iface.pluginToolBar()
        
        self.act_aeag_mask = QAction(QIcon(":plugins/mask/aeag_mask.png"), _fromUtf8("Create mask"), self.iface.mainWindow())
        self.act_aeag_mask.setCheckable(True)
        self.toolBar.addAction(self.act_aeag_mask)
        
        self.iface.addPluginToMenu("&Mask", self.act_aeag_mask)    
        
        # Add actions to the toolbar
        self.act_aeag_mask.triggered.connect(self.do_aeag_mask)
        
        #add a link to helpfile for toolbar & menu aeag , if not exist


    def unload(self):
        self.toolBar.removeAction(self.act_aeag_mask)
        self.iface.removePluginMenu("&Mask", self.act_aeag_mask)
        self.store()

    # run method that performs all the real work
    def do_aeag_mask(self, unionFeatures=True, extent=None): 
        #QMessageBox.information(None, "Cancel", "do_aeag_mask")
        self.active = True
        layermap = QgsMapLayerRegistry.instance().mapLayers()
        
        if extent == None:
            rect = self.canvas.extent()
            rect.scale(2)
        else:
            rect = extent
        
        if unionFeatures:
            g = None
            theLayer = None
            layerName = "Mask"
            for name, layer in layermap.iteritems():
                if layer.type() == QgsMapLayer.VectorLayer:
                    if layer.isValid():
                        theLayer = layer
                        features = layer.selectedFeatures()
                        for feature in features:
                            if feature.geometry() and feature.geometry().type() == QGis.Polygon:
                                if g == None:
                                    g = feature.geometry()
                                    f = feature
                                    try:
                                        layerName = "Mask " + feature.attributeMap()[layer.dataProvider().fieldNameIndex(layer.displayField())]
                                    except:
                                        layerName = "Mask"
                                else:
                                    g = g.combine(feature.geometry())
                                    layerName = "Mask"
                                    
                    layer.removeSelection()  # remove user selection after use (clean screen from yellow display)
                    
            if g != None:
                mask = QgsGeometry.fromRect(rect)
            
                toCrs = theLayer.crs();
                fromCrs = self.canvas.mapRenderer().destinationCrs()
                xform = QgsCoordinateTransform(fromCrs, toCrs)
                
                if self.canvas.hasCrsTransformEnabled():
                    mask.transform(xform)
            
                mask.makeDifference(g)
            
                maskLayer = QgsVectorLayer("MultiPolygon", layerName, "memory")
                maskLayer.setCrs(toCrs)
            
                QgsMapLayerRegistry.instance().addMapLayer(maskLayer)
            
                pr = maskLayer.dataProvider()
                fet = QgsFeature()
                fet.setGeometry(mask)
                pr.addFeatures([ fet ])
                maskLayer.updateExtents()        
            
                # style
                rendererV2 = maskLayer.rendererV2()
                for s in rendererV2.symbols():
                    s.setAlpha(0.90)
                    s.setColor(QColor(255, 255, 255))
                        
                    if isinstance(s, QgsLineSymbolV2):
                        s.setWidth(0)
                    
                # Refresh legend
                self.iface.legendInterface().refreshLayerSymbology( maskLayer ) 
                QgsMapLayerRegistry.instance().clearAllLayerCaches () #clean cache to allow mask layer to appear on refresh
                self.canvas.refresh()
               
            else:
                QMessageBox.information(None, "Attention", _fromUtf8("A selection of polygons is needed..."))

        # one mask for each feature
        else:
            layerName = "Mask"
            for name, layer in layermap.iteritems():
                if layer.type() == QgsMapLayer.VectorLayer:
                    if layer.isValid():
                        layer.select([])
                        layer.setSelectedFeatures([obj.id() for obj in layer])
                        
                        sourceProvider = layer.dataProvider()
                        
                        toCrs = layer.crs();
                        fromCrs = self.canvas.mapRenderer().destinationCrs()
                        xform = QgsCoordinateTransform(fromCrs, toCrs)

                        maskLayer = QgsVectorLayer("MultiPolygon", layerName+"_"+layer.name(), "memory")
                        maskLayer.setCrs(toCrs)
                        QgsMapLayerRegistry.instance().addMapLayer(maskLayer)
                        pr = maskLayer.dataProvider()
                        
                        # attributes
                        sourcelayer_fields = {} # looping fields with using incremented id (and not native id of fieldMap())
                        for k,v in sourceProvider.fields().iteritems():
                            sourcelayer_fields[k] = sourceProvider.fields()[k]
                        field_list=[]
                        for k,v in sourcelayer_fields.iteritems():
                            field_list.append(v)
                        pr.addAttributes( field_list ) #add all fields from target to source
                        maskLayer.updateFieldMap()

                        for feature in layer.selectedFeatures():
                            if feature.geometry().type() == QGis.Polygon:
                                g = feature.geometry()
                                
                                mask = QgsGeometry.fromRect(rect)
                                
                                if self.canvas.hasCrsTransformEnabled():
                                    mask.transform(xform)
                                
                                mask.makeDifference(g)
                            
                                fet = QgsFeature()
                                fet.setAttributeMap(feature.attributeMap())
                                fet.setGeometry(mask)
                                pr.addFeatures([ fet ])
                                    
                        # / features
                        
                        # style
                        rendererV2 = maskLayer.rendererV2()
                        for s in rendererV2.symbols():
                            s.setAlpha(0.90)
                            s.setColor(QColor(255, 255, 255))
                                
                            if isinstance(s, QgsLineSymbolV2):
                                s.setWidth(0)

                        #Refresh legend
                        layer.removeSelection()         
                        maskLayer.updateExtents()        
                        self.iface.legendInterface().refreshLayerSymbology( maskLayer ) 
                        QgsMapLayerRegistry.instance().clearAllLayerCaches () #clean cache to allow mask layer to appear on refresh

                        self.canvas.refresh()
                    else:
                        print "Invalid layer"
                else:
                    print "Selected layer is not VectorLayer"
        
        self.act_aeag_mask.setChecked(False)
        self.active = False
    
    def deactivate(self):
        self.act_aeag_mask.setChecked(False)
        self.active = False
